@@ -7,6 +7,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
+#include <string.h>
 #include <linux/kd.h>
 #include "kdmapop.h"
 #include "getfd.h"
@@ -15,12 +16,18 @@
 int
 main(int argc, char *argv[]) {
 	int fd;
+	char *console = NULL;
 
+#ifndef __klibc__
 	setlocale(LC_ALL, "");
-	bindtextdomain(PACKAGE, LOCALEDIR);
-	textdomain(PACKAGE);
+	bindtextdomain(PACKAGE_NAME, LOCALEDIR);
+	textdomain(PACKAGE_NAME);
+#endif
 
-	fd = getfd();
+	if (argc >= 3 && !strcmp(argv[1], "-C"))
+		console = argv[2];
+
+	fd = getfd(console);
 
 	return loadunimap (fd, NULL, NULL);
 }
