@@ -1,6 +1,7 @@
 #include <linux/keyboard.h>
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
 #include "ksyms.h"
 #include "nls.h"
 
@@ -770,7 +771,7 @@ static sym latin4_syms[] = {
 };
 
 static sym iso_8859_5_syms[] = { /* 160-255 */
-	{ 0x00a0, "no-break_space" },					/* 0240 */
+	{ 0x00a0, "nobreakspace" },
 	{ 0x0401, "cyrillic_capital_letter_io" },
 	{ 0x0402, "serbocroatian_cyrillic_capital_letter_dje" },
 	{ 0x0403, "macedonian_cyrillic_capital_letter_gje" },
@@ -783,7 +784,7 @@ static sym iso_8859_5_syms[] = { /* 160-255 */
 	{ 0x040a, "cyrillic_capital_letter_nje" },
 	{ 0x040b, "serbocroatian_cyrillic_capital_letter_chje" },
 	{ 0x040c, "macedonian_cyrillic_capital_letter_kje" },
-	{ 0x00ad, "soft_hyphen" },
+	{ 0x00ad, "hyphen" },
 	{ 0x040e, "bielorussian_cyrillic_capital_letter_short_u" },
 	{ 0x040f, "cyrillic_capital_letter_dze" },
 	{ 0x0410, "cyrillic_capital_letter_a" }, 			/* 0260 */
@@ -863,7 +864,7 @@ static sym iso_8859_5_syms[] = { /* 160-255 */
 	{ 0x045a, "cyrillic_small_letter_nje" },
 	{ 0x045b, "serbocroatian_cyrillic_small_letter_chje" },
 	{ 0x045c, "macedonian_cyrillic_small_letter_kje" },
-	{ 0x00a7, "paragraph_sign" },
+	{ 0x00a7, "section" },
 	{ 0x045e, "bielorussian_cyrillic_small_letter_short_u" }, 	/* printing error in ECMA-113 */
 	{ 0x045f, "cyrillic_small_letter_dze" }
 };
@@ -896,7 +897,7 @@ static sym iso_8859_7_syms[] = { /* 160-255 */
 	{ 0x0388, "Epsilonaccent" },
 	{ 0x0389, "Etaaccent" },
 	{ 0x038a, "Iotaaccent" },
-	{ 0x00bb, "rightanglequote" },
+	{ 0x00bb, "guillemotright" },
 	{ 0x038c, "Omicronaccent" },
 	{ 0x00bd, "onehalf" },
 	{ 0x038e, "Upsilonaccent" },
@@ -978,7 +979,7 @@ static sym iso_8859_8_syms[] = {
 	{ 0x00a7, "" },
 	{ 0x00a8, "" },
 	{ 0x00a9, "" },
-	{ 0x00d7, "multiplication" },
+	{ 0x00d7, "multiply" },
 	{ 0x00ab, "" },
 	{ 0x00ac, "" },
 	{ 0x00ad, "" },
@@ -1122,6 +1123,7 @@ static sym iso_8859_9_syms[] = { /* latin-5 */
 
 #include "koi8.syms.h"
 #include "cp1250.syms.h"
+#include "thai.syms.h"
 #include "ethiopic.syms.h"
 #include "sami.syms.h"
 
@@ -1481,7 +1483,8 @@ static const char *lock_syms[] = {
 	"ShiftL_Lock",
 	"ShiftR_Lock",
 	"CtrlL_Lock",
-	"CtrlR_Lock"
+	"CtrlR_Lock",
+	"CapsShift_Lock"
 };
 
 /* Keysyms whose KTYP is KT_SLOCK. */
@@ -1494,7 +1497,24 @@ static const char *sticky_syms[] = {
 	"SShiftL",
 	"SShiftR",
 	"SCtrlL",
-	"SCtrlR"
+	"SCtrlR",
+	"SCapsShift"
+};
+
+/* Keysyms whose KTYP is KT_BRL. */
+
+static const char *brl_syms[] = {
+	"Brl_blank",
+	"Brl_dot1",
+	"Brl_dot2",
+	"Brl_dot3",
+	"Brl_dot4",
+	"Brl_dot5",
+	"Brl_dot6",
+	"Brl_dot7",
+	"Brl_dot8",
+	"Brl_dot9",
+	"Brl_dot10"
 };
 
 #define E(x) { x, sizeof(x) / sizeof(x[0]) }
@@ -1512,7 +1532,9 @@ syms_entry syms[] = {
 	E(ascii_syms),		/* KT_ASCII */
 	E(lock_syms),		/* KT_LOCK */
 	{ 0, 0 },		/* KT_LETTER */
-	E(sticky_syms)		/* KT_SLOCK */
+	E(sticky_syms),		/* KT_SLOCK */
+	{ 0, 0 },		/*  */
+	E(brl_syms)		/* KT_BRL */
 };
 
 #undef E
@@ -1565,7 +1587,13 @@ synonyms[] = {
 	{ "dead_caron",         "dead_circumflex" },
 	{ "dead_breve",         "dead_tilde" },
 	{ "dead_doubleacute",   "dead_tilde" },
+/* turkish */
+	{ "Idotabove",          "Iabovedot" },
+	{ "dotlessi",           "idotless" },
 /* cyrillic */
+	{ "no-break_space",     "nobreakspace" },
+	{ "paragraph_sign",     "section" },
+	{ "soft_hyphen",        "hyphen" },
 	{ "bielorussian_cyrillic_capital_letter_i", "ukrainian_cyrillic_capital_letter_i" },
 	{ "cyrillic_capital_letter_kha", "cyrillic_capital_letter_ha" },
 	{ "cyrillic_capital_letter_ge", "cyrillic_capital_letter_ghe" },
@@ -1583,17 +1611,20 @@ synonyms[] = {
 	{ "cyrillic_small_letter_yeri", "cyrillic_small_letter_yeru" },
 	{ "cyrillic_small_letter_reversed_e", "cyrillic_small_letter_e" },
 	{ "cyrillic_small_letter_ii", "cyrillic_small_letter_i" },
-	{ "cyrillic_small_letter_short_ii", "cyrillic_small_letter_short_i" }
+	{ "cyrillic_small_letter_short_ii", "cyrillic_small_letter_short_i" },
+/* iso-8859-7 */
+	{ "rightanglequote",    "guillemotright" }
 };
 
-const int syms_size = sizeof(syms) / sizeof(syms_entry);
-const int syn_size = sizeof(synonyms) / sizeof(synonyms[0]);
+const unsigned int syms_size = sizeof(syms) / sizeof(syms_entry);
+const unsigned int syn_size = sizeof(synonyms) / sizeof(synonyms[0]);
 
 struct cs {
     const char *charset;
     sym *charnames;
     int start;
 } charsets[] = {
+    { "", NULL, 256 },
     { "iso-8859-1",	latin1_syms, 160 },
     { "iso-8859-2",	latin2_syms, 160 },
     { "iso-8859-3",	latin3_syms, 160 },
@@ -1608,19 +1639,23 @@ struct cs {
     { "cp-1250",	cp1250_syms, 128 },
     { "koi8-r",		koi8_syms, 128 },
     { "koi8-u",		koi8_syms, 128 },
+    { "tis-620",	tis_620_syms, 160 },		/* thai */
     { "iso-10646-18",	iso_10646_18_syms, 159 },	/* ethiopic */
     { "iso-ir-197",	iso_ir_197_syms, 160 },		/* sami */
     { "iso-ir-209",	iso_ir_209_syms, 160 },		/* sami */
+    /* When you add a new charset with a long (> 15 chars) name,
+     * please update the chosen_charset definition below. */
 };
 
 /* Functions for both dumpkeys and loadkeys. */
 
-static int prefer_unicode = 0;
-static const char *chosen_charset = NULL;
+int prefer_unicode = 0;
+static char chosen_charset[16] = "";
 
 void
 list_charsets(FILE *f) {
-	int i,j,lth,ct;
+	int lth,ct;
+	unsigned int i, j;
 	char *mm[] = { "iso-8859-", "koi8-" };
 
 	for (j=0; j<sizeof(mm)/sizeof(mm[0]); j++) {
@@ -1629,16 +1664,16 @@ list_charsets(FILE *f) {
 		fprintf(f, "%s{", mm[j]);
 		ct = 0;
 		lth = strlen(mm[j]);
-		for(i=0; i < sizeof(charsets)/sizeof(charsets[0]); i++) {
+		for(i=1; i < sizeof(charsets)/sizeof(charsets[0]); i++) {
 			if(!strncmp(charsets[i].charset, mm[j], lth)) {
 				if(ct++)
 					fprintf(f, ",");
-				fprintf(f, charsets[i].charset+lth);
+				fprintf(f, "%s", charsets[i].charset+lth);
 			}
 		}
 		fprintf(f, "}");
 	}
-	for(i=0; i < sizeof(charsets)/sizeof(charsets[0]); i++) {
+	for(i=1; i < sizeof(charsets)/sizeof(charsets[0]); i++) {
 		for (j=0; j<sizeof(mm)/sizeof(mm[0]); j++) {
 			lth = strlen(mm[j]);
 			if(!strncmp(charsets[i].charset, mm[j], lth))
@@ -1653,21 +1688,19 @@ list_charsets(FILE *f) {
 int
 set_charset(const char *charset) {
 	sym *p;
-	int i;
+	unsigned int i;
 
-	if (!strcasecmp(charset, "unicode")) {
-		prefer_unicode = 1;
-		return 0;
-	}
-
-	for (i = 0; i < sizeof(charsets)/sizeof(charsets[0]); i++) {
+	for (i = 1; i < sizeof(charsets)/sizeof(charsets[0]); i++) {
 		if (!strcasecmp(charsets[i].charset, charset)) {
+			charsets[0].charset = charsets[i].charset;
+			charsets[0].charnames = charsets[i].charnames;
+			charsets[0].start = charsets[i].start;
 			p = charsets[i].charnames;
 			for (i = charsets[i].start; i < 256; i++,p++) {
 				if(p->name[0])
 					syms[0].table[i] = p->name;
 			}
-			chosen_charset = charset;
+			strcpy(chosen_charset, charset);
 			return 0;
 		}
 	}
@@ -1677,44 +1710,80 @@ set_charset(const char *charset) {
 }
 
 const char *
-unicodetoksym(int code) {
-	int i, j;
+codetoksym(int code) {
+	unsigned int i;
+	int j;
 	sym *p;
 
 	if (code < 0)
 		return NULL;
-	if (code < 0x80)
-		return iso646_syms[code];
-	for (i = 0; i < sizeof(charsets)/sizeof(charsets[0]); i++) {
-		p = charsets[i].charnames;
-		for (j = charsets[i].start; j < 256; j++, p++) {
-			if (p->uni == code && p->name[0])
+
+	if (code < 0x1000) {	/* "traditional" keysym */
+		if (code < 0x80)
+			return iso646_syms[code];
+		if (KTYP(code) == KT_META)
+			return NULL;
+		if (KTYP(code) == KT_LETTER)
+			code = K(KT_LATIN, KVAL(code));
+		if (KTYP(code) > KT_LATIN)
+			return syms[KTYP(code)].table[KVAL(code)];
+
+		for (i = 0; i < sizeof(charsets)/sizeof(charsets[0]); i++) {
+			p = charsets[i].charnames;
+			if (!p)
+				continue;
+			p += KVAL(code) - charsets[i].start;
+			if (p->name[0])
 				return p->name;
 		}
 	}
+
+	else {			/* Unicode keysym */
+		code ^= 0xf000;
+
+		if (code < 0x80)
+			return iso646_syms[code];
+
+		for (i = 0; i < sizeof(charsets)/sizeof(charsets[0]); i++) {
+			p = charsets[i].charnames;
+			if (!p)
+				continue;
+			for (j = charsets[i].start; j < 256; j++, p++) {
+				if (p->uni == code && p->name[0])
+					return p->name;
+			}
+		}
+	}
+
 	return NULL;
 }
 
 /* Functions for loadkeys. */
 
-int unicode_used = 0;
-
 int
-ksymtocode(const char *s) {
-	int i;
+ksymtocode(const char *s, int direction) {
+	unsigned int i;
 	int j, jmax;
 	int keycode;
 	sym *p;
 
+	if (direction == TO_AUTO)
+		direction = prefer_unicode ? TO_UNICODE : TO_8BIT;
+
 	if (!strncmp(s, "Meta_", 5)) {
-		keycode = ksymtocode(s+5);
+		keycode = ksymtocode(s+5, TO_8BIT);
 		if (KTYP(keycode) == KT_LATIN)
 			return K(KT_META, KVAL(keycode));
+
+		/* Avoid error messages for Meta_acute with UTF-8 */
+		else if(direction == TO_UNICODE)
+		        return (0);
+
 		/* fall through to error printf */
 	}
 
 	for (i = 0; i < syms_size; i++) {
-		jmax = ((i == 0 && prefer_unicode) ? 128 : syms[i].size);
+		jmax = ((i == 0 && direction == TO_UNICODE) ? 128 : syms[i].size);
 		for (j = 0; j < jmax; j++)
 			if (!strcmp(s,syms[i].table[j]))
 				return K(i, j);
@@ -1722,23 +1791,21 @@ ksymtocode(const char *s) {
 
 	for (i = 0; i < syn_size; i++)
 		if (!strcmp(s, synonyms[i].synonym))
-			return ksymtocode(synonyms[i].official_name);
+			return ksymtocode(synonyms[i].official_name, direction);
 
-	if (prefer_unicode) {
+	if (direction == TO_UNICODE) {
 		for (i = 0; i < sizeof(charsets)/sizeof(charsets[0]); i++) {
 			p = charsets[i].charnames;
 			for (j = charsets[i].start; j < 256; j++, p++)
-				if (!strcmp(s,p->name)) {
-					unicode_used = 1;
-					return (p->uni ^ 0xf000); /* %%% */
-				}
+				if (!strcmp(s,p->name))
+					return (p->uni ^ 0xf000);
 		}
-	} else /* if (!chosen_charset) */ {
+	} else /* if (!chosen_charset[0]) */ {
 		/* note: some keymaps use latin1 but with euro,
 		   so set_charset() would fail */
 		/* note: some keymaps with charset line still use
 		   symbols from more than one character set,
-		   so we cannot have the  `if (!chosen_charset)'  here */
+		   so we cannot have the  `if (!chosen_charset[0])'  here */
 
 		for (i = 0; i < 256 - 160; i++)
 			if (!strcmp(s, latin1_syms[i].name)) {
@@ -1778,39 +1845,59 @@ ksymtocode(const char *s) {
 
 	fprintf(stderr, _("unknown keysym '%s'\n"), s);
 
-	return -1;
+	return CODE_FOR_UNKNOWN_KSYM;
 }
 
 int
-unicodetocode(int code) {
-	const char *s;
+convert_code(int code, int direction)
+{
+	const char *ksym;
+	int unicode_forced = (direction == TO_UNICODE);
+	int input_is_unicode = (code >= 0x1000);
+	int result;
 
-	s = unicodetoksym(code);
-	if (s)
-		return ksymtocode(s);
+	if (direction == TO_AUTO)
+		direction = prefer_unicode ? TO_UNICODE : TO_8BIT;
+
+	if (KTYP(code) == KT_META)
+		return code;
+	else if (!input_is_unicode && code < 0x80)
+		/* basic ASCII is fine in every situation */
+		return code;
+	else if (input_is_unicode && (code ^ 0xf000) < 0x80)
+		/* so is Unicode "Basic Latin" */
+		return code ^ 0xf000;
+	else if ((input_is_unicode && direction == TO_UNICODE) ||
+		 (!input_is_unicode && direction == TO_8BIT))
+		/* no conversion necessary */
+		result = code;
 	else {
-		unicode_used = 1;
-		return (code ^ 0xf000); /* %%% */
+		/* depending on direction, this will give us either an 8-bit
+		 * K(KTYP, KVAL) or a Unicode keysym xor 0xf000 */
+		ksym = codetoksym(code);
+		if (ksym)
+			result = ksymtocode(ksym, direction);
+		else
+			result = code;
 	}
+
+	/* if direction was TO_UNICODE from the beginning, we return the true
+	 * Unicode value (without the 0xf000 mask) */
+	if (unicode_forced && result >= 0x1000)
+		return result ^ 0xf000;
+	else
+		return result;
 }
 
 int
 add_capslock(int code)
 {
-	char buf[7];
-	const char *p;
-
-	if (KTYP(code) == KT_LATIN)
+	if (KTYP(code) == KT_LATIN && (!prefer_unicode || code < 0x80))
 		return K(KT_LETTER, KVAL(code));
-	if (KTYP(code) >= syms_size) {
-		if ((p = unicodetoksym(code ^ 0xf000)) == NULL) {
-			sprintf(buf, "U+%04x", code ^ 0xf000);
-			p = buf;
-		}
-	} else {
-		sprintf(buf, "0x%04x", code);
-		p = buf;
-	}
-	fprintf(stderr, _("plus before %s ignored\n"), p);
-	return code;
+	else if ((code ^ 0xf000) < 0x100)
+		/* Unicode Latin-1 Supplement */
+		/* a bit dirty to use KT_LETTER here, but it should work */
+		return K(KT_LETTER, code ^ 0xf000);
+	else
+		return convert_code(code, TO_AUTO);
 }
