@@ -4,6 +4,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <sysexits.h>
 #include "xmalloc.h"
 #include "nls.h"
@@ -12,6 +13,10 @@
 #include "utf8.h"
 #include "paths.h"
 
+#ifdef __klibc__
+#include "klibc_compat.h"
+#endif
+
 extern char *progname;
 
 static void
@@ -181,9 +186,9 @@ readpsffont(FILE *fontf, char **allbufp, int *allszp,
 	    int *fontwidthp, int *fontlenp, int fontpos0,
 	    struct unicode_list **uclistheadsp) {
 	char *inputbuf = NULL;
-	int inputbuflth = 0;
-	int inputlth, fontlen, fontwidth, charsize, hastable, ftoffset, utf8;
-	int i, k, n;
+	size_t inputbuflth = 0;
+	size_t inputlth, fontlen, fontwidth, charsize, hastable, ftoffset, utf8;
+	size_t i, k, n;
 
 	/*
 	 * We used to look at the length of the input file
@@ -243,22 +248,22 @@ readpsffont(FILE *fontf, char **allbufp, int *allszp,
 		utf8 = 0;
 	} else if (inputlth >= sizeof(struct psf2_header) &&
 		   PSF2_MAGIC_OK((unsigned char *)inputbuf)) {
-		struct psf2_header *psfhdr;
+		struct psf2_header psfhdr;
 		int flags;
 
-		psfhdr = (struct psf2_header *) &inputbuf[0];
+		memcpy(&psfhdr, inputbuf, sizeof(struct psf2_header));
 
-		if (psfhdr->version > PSF2_MAXVERSION) {
+		if (psfhdr.version > PSF2_MAXVERSION) {
 			char *u = _("%s: Unsupported psf version (%d)\n");
-			fprintf(stderr, u, progname, psfhdr->version);
+			fprintf(stderr, u, progname, psfhdr.version);
 			exit(EX_DATAERR);
 		}
-		fontlen = assemble_int((unsigned char *) &psfhdr->length);
-		charsize = assemble_int((unsigned char *) &psfhdr->charsize);
-		flags = assemble_int((unsigned char *) &psfhdr->flags);
+		fontlen = assemble_int((unsigned char *) &psfhdr.length);
+		charsize = assemble_int((unsigned char *) &psfhdr.charsize);
+		flags = assemble_int((unsigned char *) &psfhdr.flags);
 		hastable = (flags & PSF2_HAS_UNICODE_TABLE);
-		ftoffset = assemble_int((unsigned char *) &psfhdr->headersize);
-		fontwidth = assemble_int((unsigned char *) &psfhdr->width);
+		ftoffset = assemble_int((unsigned char *) &psfhdr.headersize);
+		fontwidth = assemble_int((unsigned char *) &psfhdr.width);
 		utf8 = 1;
 	} else
 		return -1;	/* not psf */
@@ -446,9 +451,10 @@ writepsffontheader(FILE *ofil, int width, int height, int fontlen,
 
 
 int
-writepsffont(FILE *ofil, char *fontbuf, int width, int height, int fontlen,
+writepsffont(FILE *ofil, char *fontbuf, int width, int height, size_t fontlen,
 	     int psftype, struct unicode_list *uclistheads) {
-	int bytewidth, charsize, flags, utf8, i;
+	int bytewidth, charsize, flags, utf8;
+	size_t i;
 
 	bytewidth = (width+7)/8;
 	charsize = bytewidth * height;
@@ -463,7 +469,11 @@ writepsffont(FILE *ofil, char *fontbuf, int width, int height, int fontlen,
 	writepsffontheader(ofil, width, height, fontlen, &psftype, flags);
 	utf8 = (psftype == 2);
 
-	(void)fwrite(fontbuf, charsize, fontlen, ofil);
+	if ((fwrite(fontbuf, charsize, fontlen, ofil)) != fontlen) {
+		fprintf(stderr, _("Cannot write font file"));
+		exit(EX_IOERR);
+	}
+
 	/* unimaps: -1 => do nothing: caller will append map */
 	if (uclistheads != NULL && uclistheads != (struct unicode_list*)-1) {
 		struct unicode_list *ul;
