@@ -15,15 +15,17 @@ main(int argc, char *argv[]) {
 	int fd;
 	struct { char ten, onoff; } arg;
 
+#ifndef __klibc__
 	setlocale(LC_ALL, "");
-	bindtextdomain(PACKAGE, LOCALEDIR);
-	textdomain(PACKAGE);
+	bindtextdomain(PACKAGE_NAME, LOCALEDIR);
+	textdomain(PACKAGE_NAME);
+#endif
 
 	if (argc != 2) {
 		fprintf(stderr, _("usage: %s\n"), "setvesablank ON|on|off");
 		exit(1);
 	}
-	fd = getfd();
+	fd = getfd(NULL);
 	arg.ten = 10;
 	arg.onoff = 0;
 	if (!strcmp(argv[1], "on"))
