@@ -1,8 +1,11 @@
+#include "kbd.h"
+#include "../config.h"
+#ifdef __klibc__
+#include "klibc_compat.h"
+#endif
 #include <string.h>
 #include <stdlib.h>
 
-#include "../config.h"
-
 char *progname;
 
 static inline void
@@ -13,7 +16,7 @@ set_progname(char *name) {
 	progname = (p ? p+1 : name);
 }
 
-static inline void
+static inline void attr_noreturn
 print_version_and_exit(void) {
 	printf(_("%s from %s\n"), progname, PACKAGE_STRING);
 	exit(0);
