@@ -4,12 +4,17 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sysexits.h>
+#include "kbd.h"
 #include "nls.h"
 #include "xmalloc.h"
 
+#ifdef __klibc__
+#include "klibc_compat.h"
+#endif
+
 extern char *progname;
 
-static void
+static void attr_noreturn
 nomem(void) {
 	fprintf(stderr, _("%s: out of memory\n"), progname);
 	exit(EX_OSERR);
